VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "AlignHandler"
Attribute VB_Creatable = True
Attribute VB_Exposed = True
Attribute VB_Description = "Aligner"
' ************************************************
' AlignHandler.
'
' The event handler object that performs the
' actual alignment.
' ************************************************
Option Explicit

' The environment instance.
Dim VBInstance As VBIDE.Application

' The menu command line.
Dim AlignLine As VBIDE.MenuLine

' Event connection ID.
Dim AlignID As Long

' ************************************************
' Connect the add-in to the Visual basic
' environment. This happens when the user invokes
' the Add-in Manager to put the add-in in the
' Add-Ins menu.
' ************************************************
Sub ConnectAddIn(vb_instance As VBIDE.Application)
    ' Save the Visual Basic instance for later.
    Set VBInstance = vb_instance
    
    ' Add the submenu's commands.
    Set AlignLine = VBInstance.AddInMenu.MenuItems.Add("&Align...")
    
    ' Connect event handler objects to the menu
    ' commands. Event handler objects have Public
    ' AfterClick subroutines.
    AlignID = AlignLine.ConnectEvents(Me)
End Sub

' ************************************************
' Disconnect the add-in from the Visual basic
' environment. This happens when the user invokes
' the Add-in Manager to put the add-in in the
' Add-Ins menu.
' ************************************************
Sub DisconnectAddIn(mode As Integer)
    ' Disconnect the event handler.
    AlignLine.DisconnectEvents AlignID
    
    ' Remove the menu command.
    VBInstance.AddInMenu.MenuItems.Remove AlignLine
End Sub





' ************************************************
' Get the user's alignment choices and call
' AlignControls.
' ************************************************
Public Sub AfterClick()
Dim no_controls As Boolean

    ' See if fewer than two controls are selected.
    If NoControls() Then
        no_controls = True
    ElseIf VBInstance.ActiveProject.ActiveForm. _
        SelectedControlTemplates.Count < 2 Then
            no_controls = True
    Else
            no_controls = False
    End If
    
    ' If no controls are selected, say so.
    If no_controls Then
        Beep
        MsgBox "Please select two or more controls before you start Aligner.", _
            vbOKOnly + vbInformation, _
            "No Controls Selected"
        Exit Sub
    End If
    
    ' Show the dialog and get the user's choices.
    AlignerDialog.Show vbModal

    ' If the user canceled, we're done.
    If AlignerDialog.Canceled Then Exit Sub
    
    ' Align the controls.
    AlignControls
End Sub
' ************************************************
' Return true if there are one or fewer controls
' selected.
' ************************************************
Function NoControls() As Boolean
    NoControls = True
    
    ' Sneak up on the collection of selected
    ' controls so we don't try to access an
    ' object that is Nothing.
    If VBInstance.ActiveProject Is Nothing Then _
        Exit Function
    If VBInstance.ActiveProject.ActiveForm _
        Is Nothing Then _
        Exit Function
    If VBInstance.ActiveProject.ActiveForm. _
        SelectedControlTemplates.Count < 2 Then _
        Exit Function
            
    NoControls = False
End Function

' ************************************************
' Perform the alignment.
' ************************************************
Sub AlignControls()
    ' See what kind of alignment the user wants.
    Select Case AlignerDialog.Tabs.Tab
        Case ALIGN_ROW          ' Align as a row.
            AlignRow
        Case ALIGN_COL          ' Align as a column.
            AlignCol
        Case ALIGN_ROW_AND_COL  ' Align as rows and columns.
            AlignRowsAndCols
    End Select
End Sub

' ************************************************
' Align the selected controls in a row.
' ************************************************
Sub AlignRow()
Dim the_controls As VBIDE.SelectedControlTemplates
Dim ctl As Object
Dim y As Single
Dim hgt As Single
Dim test_y As Single
Dim test_hgt As Single

    Set the_controls = VBInstance.ActiveProject _
        .ActiveForm.SelectedControlTemplates
    
    ' Find the y value to align to.
    If AlignerDialog.RowOption(ALIGN_TOP).Value Then
        y = the_controls.Item(0).Properties("Top")
        For Each ctl In the_controls
            test_y = ctl.Properties("Top")
            If y > test_y Then y = test_y
        Next ctl
            
        For Each ctl In the_controls
            ctl.Properties("Top") = y
        Next ctl
    ElseIf AlignerDialog.RowOption(ALIGN_MIDDLE).Value Then
        y = 0
        hgt = 0
        For Each ctl In the_controls
            test_y = ctl.Properties("Top")
            test_hgt = ctl.Properties("Height")
            y = y + test_y
            hgt = hgt + test_hgt
        Next ctl
            
        y = (y + hgt / 2) / the_controls.Count
        For Each ctl In the_controls
            ctl.Properties("Top") = y - _
                ctl.Properties("Height") / 2
        Next ctl
    Else
        y = the_controls.Item(0).Properties("Top") + _
            the_controls.Item(0).Properties("Height")
        For Each ctl In the_controls
            test_y = ctl.Properties("Top") + _
                     ctl.Properties("Height")
            If y < test_y Then y = test_y
        Next ctl
            
        For Each ctl In the_controls
            ctl.Properties("Top") = y - _
                ctl.Properties("Height")
        Next ctl
    End If
End Sub
' ************************************************
' Align the selected controls in a column.
' ************************************************
Sub AlignCol()
Dim the_controls As VBIDE.SelectedControlTemplates
Dim ctl As Object
Dim x As Single
Dim wid As Single
Dim test_x As Single
Dim test_wid As Single

    Set the_controls = VBInstance.ActiveProject _
        .ActiveForm.SelectedControlTemplates
    
    ' Find the x value to align to.
    If AlignerDialog.ColOption(ALIGN_LEFT).Value Then
        x = the_controls.Item(0).Properties("Left")
        For Each ctl In the_controls
            test_x = ctl.Properties("Left")
            If x > test_x Then x = test_x
        Next ctl
            
        For Each ctl In the_controls
            ctl.Properties("Left") = x
        Next ctl
    ElseIf AlignerDialog.ColOption(ALIGN_CENTER).Value Then
        x = 0
        wid = 0
        For Each ctl In the_controls
            test_x = ctl.Properties("Left")
            test_wid = ctl.Properties("Width")
            x = x + test_x
            wid = wid + test_wid
        Next ctl
            
        x = (x + wid / 2) / the_controls.Count
        For Each ctl In the_controls
            ctl.Properties("Left") = x - _
                ctl.Properties("Width") / 2
        Next ctl
    Else
        x = the_controls.Item(0).Properties("Left") + _
            the_controls.Item(0).Properties("Width")
        For Each ctl In the_controls
            test_x = ctl.Properties("Left") + _
                     ctl.Properties("Width")
            If x < test_x Then x = test_x
        Next ctl
            
        For Each ctl In the_controls
            ctl.Properties("Left") = x - _
                ctl.Properties("Width")
        Next ctl
    End If
End Sub
' ************************************************
' Align the selected controls in rows and columns.
' ************************************************
Sub AlignRowsAndCols()
Dim the_controls As VBIDE.SelectedControlTemplates
Dim controls() As VBIDE.ControlTemplate
Dim tops() As Single
Dim lefts() As Single
Dim num_rows As Integer
Dim num_cols As Integer
Dim spacing As Single
Dim i As Integer

    On Error GoTo AlignError
    
    Set the_controls = VBInstance.ActiveProject _
        .ActiveForm.SelectedControlTemplates
    
    ' Get user input values.
    num_rows = CInt(AlignerDialog.NumRowsText.Text)
    num_cols = CInt(AlignerDialog.NumColsText.Text)
    spacing = CSng(AlignerDialog.SpaceText.Text)
    If num_rows * num_cols <> the_controls.Count Then
        Beep
        MsgBox "(# Rows) * (# Cols) does not equal the number of controls you selected.", _
            vbOKOnly + vbInformation, _
            "Error Aligning Controls"
        Exit Sub
    End If
    
    ' Allocate room for the control tamplates.
    ReDim controls(0 To the_controls.Count - 1)
    ReDim tops(0 To the_controls.Count - 1)
    ReDim lefts(0 To the_controls.Count - 1)
        
    ' Copy the selected controls' templates.
    For i = 0 To the_controls.Count - 1
        Set controls(i) = the_controls(i)
        tops(i) = controls(i).Properties.Item("Top")
        lefts(i) = controls(i).Properties.Item("Left")
    Next i
            
    ' Sort the controls by rows and columns.
    SortControls num_rows, num_cols, controls, tops, lefts
    
    ' Arrange them.
    ArrangeRowColControls num_rows, num_cols, spacing, controls, tops, lefts
    Exit Sub

AlignError:
    Beep
    MsgBox "Error aligning controls." & vbCrLf & _
        Err.Description, _
        vbOKOnly + vbInformation, _
        "Error Aligning Controls"
    Exit Sub
End Sub
' ************************************************
' Arrange the controls in rows and columns.
' ************************************************
Sub ArrangeRowColControls(num_rows As Integer, num_cols As Integer, spacing As Single, controls() As VBIDE.ControlTemplate, tops() As Single, lefts() As Single)
Dim max_row As Integer
Dim max_col As Integer
Dim max_ctl As Integer
Dim same_rows As Boolean
Dim same_cols As Boolean
Dim row_align As Integer
Dim col_align As Integer
Dim row_hgt() As Single
Dim col_wid() As Single
Dim i As Integer
Dim r As Integer
Dim c As Integer
Dim start_x As Single
Dim start_y As Single
Dim x As Single
Dim y As Single
Dim val As Single

    max_row = num_rows - 1
    max_col = num_cols - 1
    max_ctl = num_rows * num_cols - 1

    ReDim row_hgt(0 To max_row)
    ReDim col_wid(0 To max_col)

    ' Get the row and column alignment values.
    For i = 0 To 2
        If AlignerDialog.RCRowOption(i).Value Then Exit For
    Next i
    row_align = i

    For i = 0 To 2
        If AlignerDialog.RCColOption(i).Value Then Exit For
    Next i
    col_align = i

    ' Find the row and column sizes.
    same_rows = (AlignerDialog.SameRowCheck.Value = vbChecked)
    If same_rows Then
        ' Make all rows the height of the
        ' tallest rectangle.
        y = controls(0).Properties.Item("Height")
        For i = 1 To max_ctl
            val = controls(i).Properties.Item("Height")
            If y < val Then y = val
        Next i
        For i = 0 To max_row
            row_hgt(i) = y
        Next i
    Else
        ' Find the height for each row.
        For r = 0 To max_row
            row_hgt(r) = controls(r * num_cols).Properties.Item("Height")
            For c = 1 To max_col
                val = controls(r * num_cols + c).Properties.Item("Height")
                If row_hgt(r) < val Then _
                   row_hgt(r) = val
            Next c
        Next r
    End If
    
    same_cols = (AlignerDialog.SameColCheck.Value = vbChecked)
    If same_cols Then
        ' Make all columns the width of the
        ' widest rectangle.
        x = controls(0).Properties.Item("Width")
        For i = 1 To max_ctl
            val = controls(i).Properties.Item("Width")
            If x < val Then x = val
        Next i
        For i = 0 To max_col
            col_wid(i) = x
        Next i
    Else
        ' Find the width for each column.
        For c = 0 To max_col
            col_wid(c) = controls(c).Properties.Item("Width")
            For r = 1 To max_row
                val = controls(c + num_cols * r).Properties.Item("Width")
                If col_wid(c) < val Then _
                   col_wid(c) = val
            Next r
        Next c
    End If
    
    ' Find the upper left corner.
    start_x = lefts(0)
    start_y = tops(0)
    For i = 1 To max_ctl
        If start_x > lefts(i) Then start_x = lefts(i)
        If start_y > tops(i) Then start_y = tops(i)
    Next i
    
    ' Position the rectangles.
    y = start_y
    
    ' Lay out the rows.
    i = 0
    For r = 0 To max_row
        x = start_x
        For c = 0 To max_col
            Select Case col_align
                Case ALIGN_LEFT
                    controls(i).Properties.Item("Left") = _
                        x
                Case ALIGN_CENTER
                    controls(i).Properties.Item("Left") = _
                        x + (col_wid(c) - _
                        controls(i).Properties.Item("Width")) / 2
                Case ALIGN_RIGHT
                    controls(i).Properties.Item("Left") = _
                        x + col_wid(c) - _
                        controls(i).Properties.Item("Width")
            End Select
            Select Case row_align
                Case ALIGN_TOP
                    controls(i).Properties.Item("Top") = _
                        y
                Case ALIGN_MIDDLE
                    controls(i).Properties.Item("Top") = _
                        y + (row_hgt(r) - _
                        controls(i).Properties.Item("Height")) / 2
                Case ALIGN_BOTTOM
                    controls(i).Properties.Item("Top") = _
                        y + row_hgt(r) - _
                        controls(i).Properties.Item("Height")
            End Select
            
            x = x + col_wid(c) + spacing
            i = i + 1
        Next c
        y = y + row_hgt(r) + spacing
    Next r
End Sub

' ************************************************
' Sort the controls according to rows and columns.
' ************************************************
Sub SortControls(num_rows As Integer, num_cols As Integer, controls() As VBIDE.ControlTemplate, tops() As Single, lefts() As Single)
Dim tmp_ctl As VBIDE.ControlTemplate
Dim max_ctl As Integer
Dim i As Integer
Dim j As Integer
Dim r As Integer
Dim min_i As Integer
Dim max_i As Integer
Dim best_value As Single
Dim best_j As Single

    ' Sort by Top value.
    max_ctl = num_rows * num_cols - 1
    For i = 0 To max_ctl - 1
        best_value = tops(i)
        best_j = i
        For j = i + 1 To max_ctl
            If best_value > tops(j) Then
                best_value = tops(j)
                best_j = j
            End If
        Next j
        If best_j <> i Then
            Set tmp_ctl = controls(i)
            Set controls(i) = controls(best_j)
            Set controls(best_j) = tmp_ctl
            tops(best_j) = tops(i)
            tops(i) = best_value
            best_value = lefts(best_j)
            lefts(best_j) = lefts(i)
            lefts(i) = best_value
        End If
    Next i
    
    ' Sort the controls in each row by Left.
    For r = 1 To num_rows
        min_i = (r - 1) * num_cols
        max_i = min_i + num_cols - 2
        For i = min_i To max_i
            best_value = lefts(i)
            best_j = i
            For j = i + 1 To max_i + 1
                If best_value > lefts(j) Then
                    best_value = lefts(j)
                    best_j = j
                End If
            Next j
            If best_j <> i Then
                Set tmp_ctl = controls(i)
                Set controls(i) = controls(best_j)
                Set controls(best_j) = tmp_ctl
                lefts(best_j) = lefts(i)
                lefts(i) = best_value
                best_value = tops(best_j)
                tops(best_j) = tops(i)
                tops(i) = best_value
            End If
        Next i
    Next r
End Sub

