Attribute VB_Name = "ScrollerFuncs"
' ************************************************
' Subroutine ArrangeScrollingArea sets properties
' for scroll bars for a scrolling area. Call it
' whenever the package control containing the
' scrolling area changes size.
' ************************************************
Option Explicit
' ************************************************
' Set properties for the scroll bars in the
' scrolling area.
' ************************************************
Sub ArrangeScrollingArea(package As PictureBox, viewport As PictureBox, hscroll As HScrollBar, vscroll As VScrollBar, scroll_area As PictureBox)
Dim viewport_xgap As Single
Dim viewport_ygap As Single
Dim need_wid As Single
Dim need_hgt As Single
Dim got_wid As Single
Dim got_hgt As Single
Dim need_hscroll As Boolean
Dim need_vscroll As Boolean

    ' See how big things are.
    viewport_xgap = viewport.Width - viewport.ScaleWidth
    viewport_ygap = viewport.Height - viewport.ScaleHeight
    got_wid = package.ScaleWidth - viewport_xgap
    got_hgt = package.ScaleHeight - viewport_ygap
    need_wid = scroll_area.Width
    need_hgt = scroll_area.Height
    
    ' See which scroll bars we need.
    If need_wid > got_wid Then
        need_hscroll = True
        got_hgt = got_hgt - hscroll.Height
    End If
    If need_hgt > got_hgt Then
        need_vscroll = True
        got_wid = got_wid - vscroll.Width
        If Not need_hscroll Then
            If need_wid > got_wid Then
                need_hscroll = True
                got_hgt = got_hgt - hscroll.Height
            End If
        End If
    End If
    
    ' Make sure we don't make things too small.
    got_wid = got_wid + viewport_xgap
    got_hgt = got_hgt + viewport_ygap
    If got_wid < vscroll.Width Then _
       got_wid = vscroll.Width
    If got_hgt < hscroll.Height Then _
       got_hgt = hscroll.Height
    
    ' Rearrange the controls.
    viewport.Move 0, 0, got_wid, got_hgt
    If need_hscroll Then _
        hscroll.Move 0, got_hgt, got_wid
    hscroll.Visible = need_hscroll
    If need_vscroll Then _
        vscroll.Move got_wid, 0, vscroll.Width, got_hgt
    vscroll.Visible = need_vscroll
    scroll_area.Move 0, 0

    ' Set the scroll bar parameters.
    If need_hscroll Then
        hscroll.Min = 0
        hscroll.Max = scroll_area.Width - viewport.ScaleWidth
        hscroll.SmallChange = 0.1 * viewport.ScaleWidth
        hscroll.LargeChange = 0.9 * viewport.ScaleWidth
    End If
    If need_vscroll Then
        vscroll.Min = 0
        vscroll.Max = scroll_area.Height - viewport.ScaleHeight
        vscroll.SmallChange = 0.1 * viewport.ScaleHeight
        vscroll.LargeChange = 0.9 * viewport.ScaleHeight
    End If
End Sub

