VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DbUser"
Attribute VB_Creatable = False
Attribute VB_Exposed = True
Option Explicit

Public UserID As String
Public MinLength As Integer
Public SuperUser As Boolean

Private TheDB As Database
Private Privileges As New Collection
Private Sub Encipher(uid As String, pwd As String, salt() As Byte, hash() As Byte)
Dim i As Integer
Dim txt(1 To 8) As Byte
Dim key(1 To 8) As Byte
    
    ' Prepare txt and key for hashing.
    PrepareTextAndKey uid, pwd, txt, key
    
    ' Mix the password and salt together.
    For i = 1 To 8
        key(i) = key(i) Xor salt(i)
    Next i
    
    ' Hash the user ID and salted password.
    OneWayHash txt, key, hash
End Sub

' ************************************************
' Read privileges for this user from the
' Privileges table.
' ************************************************
Public Sub GetPrivileges()
Dim query As String
Dim rs As Recordset

    ' Start with an empty list.
    Set Privileges = New Collection
    
    ' If there's no database or user ID, do nothing.
    If NoDb() Or UserID = "" Then Exit Sub
    
    ' Get the privileges.
    On Error GoTo GetPrivilegesError
    query = "SELECT DISTINCT privilege " & _
        "FROM GroupPrivileges, UserGroups " & _
        "WHERE userid = '" & UserID & "' AND " & _
        "GroupPrivileges.group = UserGroups.group"
    Set rs = TheDB.OpenRecordset(query, dbOpenSnapshot)
    
    ' Add the privileges to the Privileges
    ' collection.
    Do Until rs.EOF
        Privileges.Add 1, Trim$(rs!Privilege)
        rs.MoveNext
    Loop
    
    rs.Close
    Set rs = Nothing
    Exit Sub

GetPrivilegesError:
    Beep
    MsgBox "Error" & str$(Err.Number) & _
        " reading privileges." & vbCrLf & _
        vbCrLf & Err.Description, _
        vbOKOnly + vbInformation, _
        "Error Reading Privileges"
    rs.Close
    Set rs = Nothing
    Exit Sub
End Sub
' ************************************************
' Get a user ID/password pair from the user.
' Set UserID if successful.
' ************************************************
Public Sub GetUserID()
Dim uid As String
Dim pwd As String
Dim user_canceled As Boolean

    ' Make sure we have a database.
    If NoDb() Then Exit Sub

    ' Use the current user name as a default.
    GetPwdForm.UserIdText.Text = UserID
    GetPwdForm.PasswordText.Text = ""
    GetPwdForm.MinLength = MinLength
    
    ' Present the GetPwdForm dialog.
    GetPwdForm.Show vbModal
    
    ' Get the values entered.
    uid = GetPwdForm.UserIdText.Text
    pwd = GetPwdForm.PasswordText.Text
    user_canceled = GetPwdForm.Canceled
    Unload GetPwdForm
    
    ' If the user canceled, do nothing more.
    If user_canceled Then Exit Sub
    
    ' If the user ID is valid, use it.
    If ValidPassword(uid, pwd) Then
        UserID = uid
    Else
        Beep
        MsgBox "Invalid user ID/password pair.", _
            vbOKOnly + vbInformation, _
            "Invalid Password"
    End If
End Sub
' ************************************************
' Perform a 2-PHT transformation on a1 and a2 to
' obtain b1 and b2.
' ************************************************
Private Sub DoPHT(a1 As Integer, a2 As Integer, b1 As Integer, b2 As Integer)
    b1 = (2 * a1 + a2) Mod 256
    b2 = (a1 + a2) Mod 245
End Sub


' ************************************************
' Return true if the user has this privilege.
' ************************************************
Property Get HasPrivilege(priv As String) As Boolean
    On Error GoTo PrivError
    HasPrivilege = (Privileges.Item(priv) = 1)
    Exit Property
    
PrivError:
    HasPrivilege = False
    Exit Property
End Property

' ************************************************
' Save the database for later use.
' ************************************************
Public Sub Initialize(db As Database)
    Set TheDB = db
End Sub

' ************************************************
' Warn the user and return true if there is no
' database available.
' ************************************************
Private Function NoDb() As Boolean
    If TheDB Is Nothing Then
        Beep
        MsgBox "No database available.", _
            vbOKOnly + vbInformation, _
            "No Database"
        NoDb = True
    Else
        NoDb = False
    End If
End Function

' ************************************************
' Return a one-way hashing of plaintext txt using
' key value key. Both txt and key are repeated if
' necessary so they are at least 8 characters long.
' ************************************************
Private Sub OneWayHash(txt() As Byte, key() As Byte, result() As Byte)
Const NUM_ROUNDS = 8
Const NUM_KEYS = 2 * NUM_ROUNDS + 1

Dim key_len As Integer
Dim i As Integer
Dim round As Integer
Dim C(1 To NUM_KEYS, 1 To 8) As Byte
Dim keys(1 To NUM_KEYS, 1 To 8) As Byte
Dim shifted(1 To 8) As Byte
Dim log45 As Double
Dim rslt(1 To 8) As Integer
Dim tmp As Integer
Dim pht(1 To 2, 1 To 8) As Integer
Dim power As Integer
Dim p As Integer

    ' Prepare the round constants C.
    For round = 1 To NUM_KEYS
        For i = 1 To 8
            power = (9 * round + i) Mod 256
            tmp = 1
            For p = 1 To power
                tmp = (tmp * 45) Mod 257
            Next p
            power = tmp
            tmp = 1
            For p = 1 To power
                tmp = (tmp * 45) Mod 257
            Next p
            C(round, i) = tmp
        Next i
    Next round
    
    ' Prepare the subkeys.
    For i = 1 To 8
        keys(1, i) = key(i)
        shifted(i) = key(i)
    Next i
    For round = 2 To NUM_KEYS
        For i = 1 To 8
            ' Shift the subkey.
            shifted(i) = _
                ((shifted(i) And 31) * 8) + _
                shifted(i) \ 32
            keys(round, i) = (CInt(shifted(i)) + _
                C(round, i)) Mod 256
        Next i
    Next round
    
    ' Copy the plaintext into the result.
    For i = 1 To 8
        rslt(i) = txt(i)
    Next i
    
    ' Perform the encryption.
    log45 = Log(45#)
    For round = 1 To NUM_ROUNDS
        ' XOR or ADD.
        For i = 1 To 8
            If i = 1 Or i = 4 Or i = 5 Or i = 8 Then
                ' Xor.
                tmp = rslt(i) Xor keys(2 * round - 1, i)
                ' 45^x.
                If tmp = 0 Then
                    rslt(i) = 0
                Else
                    power = tmp
                    tmp = 1
                    For p = 1 To power
                        tmp = (tmp * 45) Mod 257
                    Next p
                    rslt(i) = tmp
                End If
                ' Add.
                rslt(i) = rslt(i) + keys(2 * round, i)
            Else
                ' Add.
                tmp = rslt(i) + keys(2 * round - 1, i)
                ' Log.
                If tmp = 0 Then
                    rslt(i) = 0
                Else
                    rslt(i) = Log(tmp) / log45
                End If
                ' Xor.
                rslt(i) = rslt(i) Xor keys(2 * round, i)
            End If
        Next i
        
        ' Perform 2-PHT transformations.
        DoPHT rslt(1), rslt(2), pht(1, 1), pht(1, 5)
        DoPHT rslt(3), rslt(4), pht(1, 2), pht(1, 6)
        DoPHT rslt(5), rslt(6), pht(1, 3), pht(1, 7)
        DoPHT rslt(7), rslt(8), pht(1, 4), pht(1, 8)
        
        DoPHT pht(1, 1), pht(1, 2), pht(2, 1), pht(2, 5)
        DoPHT pht(1, 3), pht(1, 4), pht(2, 2), pht(2, 6)
        DoPHT pht(1, 5), pht(1, 6), pht(2, 3), pht(2, 7)
        DoPHT pht(1, 7), pht(1, 8), pht(2, 4), pht(2, 8)
        
        DoPHT pht(2, 1), pht(2, 2), rslt(1), rslt(2)
        DoPHT pht(2, 3), pht(2, 4), rslt(3), rslt(4)
        DoPHT pht(2, 5), pht(2, 6), rslt(5), rslt(6)
        DoPHT pht(2, 7), pht(2, 8), rslt(7), rslt(8)
    Next round
    
    ' Perform the output transformation.
    For i = 1 To 8
        If i = 1 Or i = 4 Or i = 5 Or i = 8 Then
            ' Xor.
            tmp = rslt(i) Xor keys(2 * round - 1, i)
            rslt(i) = tmp
        Else
            ' Add.
            tmp = rslt(i) + keys(2 * round - 1, i)
            rslt(i) = tmp Mod 256
        End If
    Next i
    
    ' Copy the result into the result array.
    For i = 1 To 8
        result(i) = rslt(i) Mod 256
    Next i
End Sub


' ************************************************
' Allow the user to update a password. Update
' UserID and return true if successful.
' ************************************************
Public Function UpdatePassword() As Boolean
Dim uid As String
Dim oldpwd As String
Dim newpwd As String
Dim user_canceled As Boolean

    ' Assume we will fail.
    UpdatePassword = False
    
    ' Make sure we have a database.
    If NoDb() Then Exit Function

    ' Use the current user name as a default.
    SetPwdForm.UserIdText.Text = UserID
    SetPwdForm.PasswordText.Text = ""
    SetPwdForm.NewText.Text = ""
    SetPwdForm.VerifyText.Text = ""
    SetPwdForm.MinLength = MinLength
    If SuperUser Then
        SetPwdForm.PasswordLabel.Enabled = False
        SetPwdForm.PasswordText.Enabled = False
    End If
    
    ' Present the SetPwdForm dialog.
    SetPwdForm.Show vbModal
    
    ' Get the values entered.
    uid = SetPwdForm.UserIdText.Text
    oldpwd = SetPwdForm.PasswordText.Text
    newpwd = SetPwdForm.NewText.Text
    user_canceled = SetPwdForm.Canceled
    Unload SetPwdForm
    
    ' If the user canceled, do nothing more.
    If user_canceled Then Exit Function
    
    ' If the user ID is invalid, do no more.
    If Not SuperUser Then
        If Not ValidPassword(uid, oldpwd) Then
            Beep
            MsgBox "Invalid user ID/password pair.", _
                vbOKOnly + vbInformation, _
                "Invalid Password"
            Exit Function
        End If
    End If

    ' Update the password.
    UserID = uid
    UpdatePassword = SetPassword(newpwd)
End Function

' ************************************************
' Prepare the user ID and password for hashing.
' ************************************************
Private Sub PrepareTextAndKey(ByVal uid As String, ByVal pwd As String, txt() As Byte, key() As Byte)
Const PADDING = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdef"
Dim i As Integer
    
    ' Add letters ABC...Zabc... to make the user
    ' ID and password at least 32 characters long.
    uid = uid & PADDING
    pwd = pwd & PADDING
    
    ' Xor the strings down to 8 characters.
    For i = 1 To 8
        txt(i) = Asc(Mid$(uid, i, 1)) Xor _
                 Asc(Mid$(uid, i + 8, 1)) Xor _
                 Asc(Mid$(uid, i + 16, 1)) Xor _
                 Asc(Mid$(uid, i + 24, 1))
        key(i) = Asc(Mid$(pwd, i, 1)) Xor _
                 Asc(Mid$(pwd, i + 8, 1)) Xor _
                 Asc(Mid$(pwd, i + 16, 1)) Xor _
                 Asc(Mid$(pwd, i + 24, 1))
    Next i
End Sub

' ************************************************
' Return true if the user ID/password combination
' is valid.
' ************************************************
Private Function ValidPassword(uid As String, pwd As String) As Boolean
Dim query As String
Dim rs As Recordset
Dim i As Integer
Dim salt(1 To 8) As Byte
Dim hash(1 To 8) As Byte

    ' Assume we will fail.
    ValidPassword = False
    
    On Error GoTo ValidPasswordError
    
    ' Get the salt and hashed values.
    query = "SELECT " & _
        "salt1, salt2, salt3, salt4, " & _
        "salt5, salt6, salt7, salt8, " & _
        "hash1, hash2, hash3, hash4, " & _
        "hash5, hash6, hash7, hash8 " & _
        "FROM Passwords WHERE userid = '" & _
        uid & "'"
    Set rs = TheDB.OpenRecordset(query, dbOpenSnapshot)
    
    ' If we didn't find it, it's not a valid ID.
    If rs.EOF Then GoTo ValidPasswordDone
    
    ' Copy the salt into the salt array.
    For i = 1 To 8
        salt(i) = rs.Fields(i - 1)
    Next i
    
    ' Encipher the uid using the key and salt.
    Encipher uid, pwd, salt, hash
    
    ' Verify that this is the correct hash.
    For i = 1 To 8
        If hash(i) <> rs.Fields(i + 7) Then _
            GoTo ValidPasswordDone
    Next i
    
    ' This user ID/password is correct.
    ValidPassword = True

ValidPasswordDone:
    ' Finish up.
    rs.Close
    Set rs = Nothing
    Exit Function

ValidPasswordError:
    Beep
    MsgBox "Error" & str$(Err.Number) & _
        " validating password." & vbCrLf & _
        vbCrLf & Err.Description, _
        vbOKOnly + vbInformation, _
        "Error Validating Password"
    ValidPassword = False
    rs.Close
    Set rs = Nothing
    Exit Function
End Function

' ************************************************
' Set the user's password, creating the user ID if
' it does not already exist. Return true if
' successful.
' ************************************************
Private Function SetPassword(pwd As String) As Boolean
Dim i As Integer
Dim salt(1 To 8) As Byte
Dim hash(1 To 8) As Byte
Dim query As String
Dim qd As QueryDef

    On Error GoTo SetPasswordError
    
    If NoDb() Then Exit Function
    
    ' Assume we will fail.
    SetPassword = False
    
    ' Generate the salt values.
    Randomize
    For i = 1 To 8
        salt(i) = CByte(256 * Rnd)
    Next i
    
    ' Encipher the UserID using the key and salt.
    Encipher UserID, pwd, salt, hash
    
    ' Update the database entry.
    query = "UPDATE Passwords SET "
    For i = 1 To 8
        query = query & "salt" & Format$(i) & _
            " =" & str$(salt(i)) & ","
    Next i
    For i = 1 To 8
        query = query & "hash" & Format$(i) & _
            " =" & str$(hash(i))
        If i < 8 Then query = query & ","
    Next i
    query = query & " WHERE userid = '" & _
        UserID & "'"
    
    Set qd = TheDB.CreateQueryDef("", query)
    qd.Execute
    
    ' See if we updated a row.
    If qd.RecordsAffected < 1 Then
        ' We did not. Insert a new record.
        query = "INSERT INTO Passwords VALUES ('" & _
            UserID & "'"
        For i = 1 To 8
            query = query & "," & str$(salt(i))
        Next i
        For i = 1 To 8
            query = query & "," & str$(hash(i))
        Next i
        query = query & ")"
        
        Set qd = TheDB.CreateQueryDef("", query)
        qd.Execute
        
        ' If we did not create the row, we failed.
        If qd.RecordsAffected < 1 Then
            SetPassword = False
            qd.Close
            Set qd = Nothing
            Beep
            MsgBox "Unable to create password entry.", _
                vbOKOnly + vbInformation, _
                "Error Changing Password"
            Exit Function
        End If
    End If
        
    SetPassword = True
    qd.Close
    Set qd = Nothing
    Exit Function
    
SetPasswordError:
    Beep
    MsgBox "Error" & str$(Err.Number) & _
        " changing password." & vbCrLf & vbCrLf & _
        Err.Description, _
        vbOKOnly + vbInformation, _
        "Error Changing Password"
    qd.Close
    Set qd = Nothing
    Exit Function
End Function
Private Sub Class_Initialize()
    UserID = ""
    MinLength = 0
End Sub


Private Sub Class_Terminate()
    Set TheDB = Nothing
End Sub


